@@ -227,7 +227,7 @@ A prefix argument SET-LINE forces loading but only up to the current line."
            (lambda (_condition)
              (when (member 'warnings-tree idris-warnings-printing)
                (idris-list-compiler-notes))))))
-    (user-error "Cannot find file for current buffer")))
+    (error "Cannot find file for current buffer")))
 
 (defun idris-view-compiler-log ()
   "Jump to the log buffer, if it is open."
@@ -244,7 +244,7 @@ A prefix argument SET-LINE forces loading but only up to the current line."
                                 #'(lambda (w1 w2) (<= (overlay-start w1) (overlay-start w2))))))
     (if warnings-forward
         (goto-char (overlay-start (car warnings-forward)))
-      (user-error "No warnings or errors until end of buffer"))))
+      (error "No warnings or errors until end of buffer"))))
 
 (defun idris-previous-error ()
   "Jump to the previous error overlay in the buffer."
@@ -253,7 +253,7 @@ A prefix argument SET-LINE forces loading but only up to the current line."
                                  #'(lambda (w1 w2) (>= (overlay-end w1) (overlay-end w2))))))
     (if warnings-backward
         (goto-char (overlay-end (car warnings-backward)))
-      (user-error "No warnings or errors until beginning of buffer"))))
+      (error "No warnings or errors until beginning of buffer"))))
 
 (defun idris-load-file-sync ()
   "Pass the current buffer's file synchronously to the inferior Idris process.
@@ -280,7 +280,7 @@ This sets the load position to point, if there is one."
             (setq idris-currently-loaded-buffer (current-buffer))
             (idris-make-clean)
             (idris-update-loaded-region (car result)))))
-    (user-error "Cannot find file for current buffer")))
+    (error "Cannot find file for current buffer")))
 
 
 
@@ -512,9 +512,9 @@ Useful for writing papers or slides."
         (fmt (completing-read "What format? " '("html", "latex") nil t nil nil "latex"))
         (width (read-string "How wide? " nil nil "80")))
     (if (<= (string-to-number width) 0)
-        (user-error "Width must be positive")
+        (error "Width must be positive")
       (if (< (length what) 1)
-          (user-error "Nothing to pretty-print")
+          (error "Nothing to pretty-print")
         (let ((text (idris-eval `(:interpret ,(concat ":pprint " fmt " " width " " what)))))
           (with-idris-info-buffer
             (insert (car text))
@@ -773,7 +773,7 @@ A plain prefix ARG causes the command to prompt for hints and recursion
                            (idris-eval `(:proof-search ,(cdr what) ,(car what) ,hints ,@depth))
                            ))))
         (if (string= result "")
-            (user-error "Nothing found")
+            (error "Nothing found")
           (idris-replace-hole-with result))))))
 
 (defun idris-proof-search-next ()
@@ -781,7 +781,7 @@ A plain prefix ARG causes the command to prompt for hints and recursion
 Idris 2 only."
   (interactive)
   (if (not proof-region-start)
-      (user-error "You must proof search first before looking for subsequent proof results")
+      (error "You must proof search first before looking for subsequent proof results")
     (let ((result (car (idris-eval `:proof-search-next))))
       (if (string= result "No more results")
           (message "No more results")
@@ -805,11 +805,11 @@ Idris 2 only."
             final-point
             (prefix (idris-line-indentation-for what)))
         (if (string= result "")
-            (user-error "Nothing found")
-          (beginning-of-line)
+            (error "Nothing found")
+          (goto-char (line-beginning-position))
           (forward-line)
           (while (and (not (eobp))
-                      (progn (beginning-of-line)
+                      (progn (goto-char (line-beginning-position))
                              (looking-at-p (concat prefix "\\s-+"))))
             (forward-line))
           (insert prefix)
@@ -818,14 +818,20 @@ Idris 2 only."
           (insert result)
           (setq def-region-end (point))
           (newline)
-          (goto-char final-point))))))
+          (goto-char final-point)
+;          (save-excursion
+;            (forward-line 1)
+;            (setq def-region-start (point))
+;            (insert result)
+;            (setq def-region-end (point)))
+          )))))
 
 (defun idris-generate-def-next ()
   "Replace the previous generated definition with next definition, if it exists.
 Idris 2 only."
   (interactive)
   (if (not def-region-start)
-      (user-error "You must program search first before looking for subsequent program results")
+      (error "You must program search first before looking for subsequent program results")
     (let ((result (car (idris-eval `:generate-def-next))))
       (if (string= result "No more results")
           (message "No more results")
@@ -841,7 +847,7 @@ Idris 2 only."
   (interactive)
   (let ((what (idris-thing-at-point)))
     (unless (car what)
-      (user-error "Could not find a hole at point to refine by"))
+      (error "Could not find a hole at point to refine by"))
     (idris-load-file-sync)
     (let ((results (car (idris-eval `(:intro ,(cdr what) ,(car what))))))
       (pcase results
@@ -853,7 +859,7 @@ Idris 2 only."
   (interactive "MRefine by: ")
   (let ((what (idris-thing-at-point)))
     (unless (car what)
-      (user-error "Could not find a hole at point to refine by"))
+      (error "Could not find a hole at point to refine by"))
     (idris-load-file-sync)
     (let ((result (car (idris-eval `(:refine ,(cdr what) ,(car what) ,name)))))
       (idris-replace-hole-with result))))
@@ -1015,7 +1021,7 @@ performed silently without confirmation from the user."
            (ibc (concat (file-name-sans-extension fname) ".ibc")))
       (if (not (member (file-name-extension fname)
                        '("idr" "lidr" "org" "markdown" "md")))
-          (user-error "The current file is not an Idris file")
+          (error "The current file is not an Idris file")
         (when (or no-confirmation (y-or-n-p (concat "Really delete " ibc "?")))
           (when (file-exists-p ibc)
             (delete-file ibc)
@@ -1027,7 +1033,7 @@ performed silently without confirmation from the user."
 It is an error if POS is not in the specified term. TERM should
 be Idris's own serialization of the term in question."
   (unless (equal (get-char-property pos 'idris-tt-term) term)
-    (user-error "Term not present at %s" pos))
+    (error "Term not present at %s" pos))
   (save-excursion
     ;; Find the beginning of the active term
     (goto-char pos)
@@ -1260,7 +1266,7 @@ of the term to replace."
   (interactive)
   (let ((files (idris-find-file-upwards "ipkg")))
     (cond ((= (length files) 0)
-           (user-error "No .ipkg file found"))
+           (error "No .ipkg file found"))
           ((= (length files) 1)
            (find-file (car files)))
           (t (find-file (completing-read "Package file: " files nil t))))))
@@ -1280,7 +1286,7 @@ of the term to replace."
          (first-mod (read-string
                      (format "First module name (%s): " module-name-suggestion)
                      nil nil module-name-suggestion)))
-    (when (file-exists-p create-in) (user-error "%s already exists" create-in))
+    (when (file-exists-p create-in) (error "%s already exists" create-in))
     (when (string= src-dir "") (setq src-dir nil))
     (make-directory create-in t)
     (when src-dir (make-directory (concat (file-name-as-directory create-in) src-dir) t))
