@@ -34,9 +34,35 @@
 (require 'idris-common-utils)
 (require 'idris-prover)
 (require 'idris-highlight-input)
+(require 'idris-xref)
 
-(eval-when-compile (require 'cl))
+(eval-when-compile (require 'cl-lib))
 
+;;; Words of encouragement - strongly inspired by Slime
+(defun idris-user-first-name ()
+  (let ((name (if (string= (user-full-name) "")
+                  (user-login-name)
+                (user-full-name))))
+    (string-match "^[^ ]*" name)
+    (capitalize (match-string 0 name))))
+
+(defvar idris-words-of-encouragement
+  `("Let the hacking commence!"
+    "Hacks and glory await!"
+    "Hack and be merry!"
+    ,(format "%s, this could be the start of a beautiful program."
+             (idris-user-first-name))
+    ,(format "%s, this could be the start of a beautiful proof."
+             (idris-user-first-name))
+    "The terms have seized control of the means of computation - a glorious future awaits!"
+    "It typechecks! Ship it!"
+    "Do you know 'Land of My Fathers'?"
+    "Constructors are red / Types are blue / Your code always works / Because Idris loves you"))
+
+(defun idris-random-words-of-encouragement ()
+  "Return a random string of encouragement"
+  (nth (random (length idris-words-of-encouragement))
+       idris-words-of-encouragement))
 
 (defvar idris-prompt-string "Idris"
   "The prompt shown in the REPL.")
@@ -55,7 +81,7 @@
   "Welcome to the Idris REPL!")
 
 (defun idris-repl-get-logo ()
-  "Return the path to the Idris logo if it exists, or `nil' if not."
+  "Return the path to the Idris logo if it exists, or nil if not."
   (let ((logo-path (concat idris-mode-path "logo-small.png")))
     (if (file-readable-p logo-path)
         logo-path
@@ -63,7 +89,7 @@
 
 (defun idris-repl-insert-logo ()
   "Attempt to insert a graphical logo.
-Returns non-`nil' on success, `nil' on failure."
+Returns non-nil on success, nil on failure."
   (let ((logo (idris-repl-get-logo)))
     (if (and (display-graphic-p)
              (image-type-available-p 'png)
@@ -75,13 +101,13 @@ Returns non-`nil' on success, `nil' on failure."
 
 (defun idris-repl-animate-banner ()
   "Insert a text banner using animation.
-Returns non-`nil' on success, `nil' on failure."
+Returns non-nil on success, nil on failure."
   (animate-string (idris-repl-welcome-message) 0 0)
   t)
 
 (defun idris-repl-text-banner ()
   "Insert a text banner with no animation.
-Returns non-`nil' on success, `nil' on failure."
+Returns non-nil on success, nil on failure."
   (insert (idris-repl-welcome-message))
   t)
 
@@ -96,7 +122,9 @@ Returns non-`nil' on success, `nil' on failure."
                  version-string)
         (insert (propertize (concat "Idris " version-string)
                             'face 'italic)
-                "\n\n")))))
+                "\n")))
+    (when idris-display-words-of-encouragement
+      (insert (idris-random-words-of-encouragement) "\n"))))
 
 (defun idris-repl-insert-prompt (&optional always-insert)
   "Insert or update Idris prompt in buffer.
@@ -124,8 +152,9 @@ If ALWAYS-INSERT is non-nil, always insert a prompt at the end of the buffer."
       (let ((inhibit-read-only t))
         (insert prompt)))
     (set-marker idris-input-start (point-max))
-    (goto-char idris-input-start)))
-
+    (goto-char idris-input-start)
+    (and (get-buffer-window)
+         (set-window-point (get-buffer-window) idris-input-start))))
 
 (defun idris-repl-update-prompt (new-prompt)
   "Update prompt string to NEW-PROMPT."
@@ -141,7 +170,7 @@ If ALWAYS-INSERT is non-nil, always insert a prompt at the end of the buffer."
       (let ((buffer (get-buffer-create idris-repl-buffer-name)))
         (save-selected-window
           (when idris-repl-show-repl-on-startup
-            (pop-to-buffer buffer t))
+            (display-buffer buffer t))
           (with-current-buffer buffer
             (idris-repl-mode)
             (idris-repl-buffer-init))
@@ -157,17 +186,20 @@ If ALWAYS-INSERT is non-nil, always insert a prompt at the end of the buffer."
       (idris-repl-insert-prompt)
       (insert current-input))))
 
-(defun idris-switch-to-output-buffer ()
+(defun idris-switch-to-repl ()
   "Select the output buffer and scroll to bottom."
   (interactive)
   (pop-to-buffer (idris-repl-buffer))
   (goto-char (point-max)))
 
+(define-obsolete-function-alias 'idris-switch-to-output-buffer 'idris-switch-to-repl "2022-12-28")
+
+(autoload 'idris-run "idris-commands.el")
 ;;;###autoload
 (defun idris-repl ()
   (interactive)
   (idris-run)
-  (idris-switch-to-output-buffer))
+  (idris-switch-to-repl))
 
 (defvar idris-repl-mode-map
   (let ((map (make-sparse-keymap)))
@@ -187,11 +219,14 @@ If ALWAYS-INSERT is non-nil, always insert a prompt at the end of the buffer."
                   idris-define-general-keys
                   idris-define-active-term-keys)
              do (funcall keyer map))
+    (substitute-key-definition 'idris-switch-to-repl
+                               'idris-switch-to-last-idris-buffer
+                               map)
     map)
   "Keymap used in Idris REPL mode.")
 
 (easy-menu-define idris-repl-mode-menu idris-repl-mode-map
-  "Menu for the Idris REPL mode"
+  "Menu for the Idris REPL mode."
   `("Idris REPL"
     ("Interpreter options" :active idris-process
      ["Show implicits" (idris-set-option :show-implicits t)
@@ -204,19 +239,19 @@ If ALWAYS-INSERT is non-nil, always insert a prompt at the end of the buffer."
       :visible (idris-get-option :error-context)])
     ["Show term interaction widgets" idris-add-term-widgets t]
     ["Customize idris-mode" (customize-group 'idris) t]
-    ["Quit inferior idris process" idris-quit t]
+    ["Quit inferior Idris process" idris-quit t]
     ))
 
 (define-derived-mode idris-repl-mode fundamental-mode "Idris-REPL"
   "Major mode for interacting with Idris.
-    \\{idris-repl-mode-map}
+\\{idris-repl-mode-map}
 Invokes `idris-repl-mode-hook'."
                                         ;syntax-table?
   :group 'idris-repl
   (set (make-local-variable 'indent-tabs-mode) nil)
   (add-hook 'idris-event-hooks 'idris-repl-event-hook-function)
   (add-hook 'kill-buffer-hook 'idris-repl-remove-event-hook-function nil t)
-  (when idris-repl-history-file
+  (when (idris-repl-history-file-f)
     (idris-repl-safe-load-history)
     (add-hook 'kill-buffer-hook
               'idris-repl-safe-save-history nil t))
@@ -224,7 +259,8 @@ Invokes `idris-repl-mode-hook'."
   (set (make-local-variable 'completion-at-point-functions) '(idris-repl-complete))
   (setq mode-name `("Idris-REPL" (:eval (if idris-rex-continuations "!" ""))))
   (set (make-local-variable 'prop-menu-item-functions)
-       '(idris-context-menu-items)))
+       '(idris-context-menu-items))
+  (add-hook 'xref-backend-functions #'idris-xref-backend nil 'local))
 
 (defun idris-repl-remove-event-hook-function ()
   (setq idris-prompt-string "Idris")
@@ -240,7 +276,22 @@ Invokes `idris-repl-mode-hook'."
      t)
     (`(:warning ,output ,_target)
      (when (member 'warnings-repl idris-warnings-printing)
-       (idris-repl-write-string (format "Error: %s line %d (col %d):\n%s" (nth 0 output) (nth 1 output) (if (eq (safe-length output) 3) 0 (nth 2 output)) (car (last output))))))
+       (pcase output
+         (`(,fname (,start-line ,start-col) (,_end-line ,_end-col) ,msg ,_spans)
+          (idris-repl-write-string (format "Error: %s line %d (col %d):\n%s"
+                                           fname
+                                           (1+ start-line)
+                                           (1+ start-col)
+                                           msg)))
+         ;; Keeping this old format for backward compatibility.
+         ;; Probably we can remove it at some point.
+         (_ (idris-repl-write-string (format "Error: %s line %d (col %d):\n%s"
+                                             (nth 0 output)
+                                             (nth 1 output)
+                                             (if (eq (safe-length output) 3)
+                                                 0
+                                               (nth 2 output))
+                                             (car (last output))))))))
     (`(:run-program ,file ,_target)
      (idris-execute-compiled-program file))
     (_ nil)))
@@ -281,28 +332,17 @@ Invokes `idris-repl-mode-hook'."
       (idris-repl-eval-string input input-start))))
 
 (defun idris-repl-complete ()
-  "Completion of the current input"
-  (let* ((input (idris-repl-current-input))
-         (result (idris-eval `(:repl-completions ,input))))
-    (cl-destructuring-bind (completions partial) (car result)
-      (if (null completions)
-          nil
-        (list (+ idris-input-start (length partial)) (point-max) completions)))))
-
-(defun find-common-prefix (input slist)
-  "Finds longest common prefix of all strings in list."
-  (let ((first (car slist))
-        (ilen (length input)))
-    (if (> (length first) ilen)
-        (progn
-          (let ((next (substring first 0 (1+ ilen))))
-            (if (cl-every (lambda (p) (string-prefix-p next p)) slist)
-                (find-common-prefix next slist)
-              input)))
-      input)))
+  "Completion of the current input."
+  (when idris-completion-via-compiler
+    (let* ((input (idris-repl-current-input))
+           (result (idris-eval `(:repl-completions ,input))))
+      (cl-destructuring-bind (completions partial) (car result)
+        (if (null completions)
+            nil
+          (list (+ idris-input-start (length partial)) (point-max) completions))))))
 
 (defun idris-repl-begin-of-prompt ()
-  "Got to the beginning of linke or the prompt."
+  "Go to the beginning of line or the prompt."
   (interactive)
   (cond ((and (>= (point) idris-input-start)
               (idris-same-line-p (point) idris-input-start))
@@ -314,8 +354,10 @@ Invokes `idris-repl-mode-hook'."
   (buffer-substring-no-properties idris-input-start (point-max)))
 
 (defun idris-repl-highlight-input (start-pos start-line start-col end-line end-col props)
-  "Apply semantic highlighting to the REPL input beginning at START-POS using the Idris location information START-LINE, START-COL, END-LINE, and END-COL and semantic annotations PROPS."
-  (let ((buffer (get-buffer (idris-buffer-name :repl))))
+  "Apply semantic highlighting to the REPL input beginning at START-POS using
+the Idris location information START-LINE, START-COL, END-LINE, and END-COL
+and semantic annotations PROPS."
+  (let ((buffer (get-buffer idris-repl-buffer-name)))
     (with-current-buffer buffer
       (save-restriction
         (widen)
@@ -330,8 +372,7 @@ Invokes `idris-repl-mode-hook'."
                (start-col-repl (+ input-col start-col))
                (end-line-repl (+ input-line end-line -1))
                (end-col-repl (+ input-col end-col)))
-          (idris-highlight-input-region buffer
-                                        start-line-repl start-col-repl
+          (idris-highlight-input-region start-line-repl start-col-repl
                                         end-line-repl end-col-repl
                                         props))))))
 
@@ -352,7 +393,7 @@ Invokes `idris-repl-mode-hook'."
                  ,props)
                (idris-repl-highlight-input
                 start start-line start-col end-line end-col props))))))
-       (_ (idris-repl-insert-result result spans)))) ;; The actual result
+       (_ (idris-repl-insert-result (or result "") spans)))) ;; The actual result
     ((:error condition &optional spans)
      (idris-repl-show-abort condition spans))))
 
@@ -384,7 +425,7 @@ Invokes `idris-repl-mode-hook'."
 
 
 (defun idris-repl-insert-result (string &optional highlighting)
-  "Insert STRING and mark it asg evaluation result.
+  "Insert STRING and mark it as evaluation result.
 Optional argument HIGHLIGHTING is a collection of semantic
 highlighting information from Idris."
   (with-current-buffer (idris-repl-buffer)
@@ -423,7 +464,7 @@ highlighting information from Idris."
   "History list of strings entered into the REPL buffer.")
 
 (defun idris-repl-add-to-input-history (string)
-  "Adds input to history."
+  "Add input STRING to history."
   (unless (equal string "")
     (setq idris-repl-input-history
           (remove string idris-repl-input-history)))
@@ -444,7 +485,7 @@ highlighting information from Idris."
 
 (defun idris-repl-history-replace (direction)
   "Replace the current input with the next line in DIRECTION.
-DIRECTION is 'forward' or 'backward' (in the history list)."
+DIRECTION is `forward' or `backward' (in the history list)."
   (let* ((min-pos -1)
          (max-pos (length idris-repl-input-history))
          (prefix (idris-repl-history-prefix))
@@ -531,34 +572,44 @@ The handler will use qeuery to ask the use if the error should be ingored."
          nil
        (signal (car err) (cdr err))))))
 
+(defun idris-repl-history-file-f ()
+  "Return repl history file.
+
+Use `idris-repl-history-file' if set or fallback
+ to filepath computed from the `idris-interpreter-path'."
+  (or idris-repl-history-file
+      (idris-file-name-concat "~"
+                              (concat "." (file-name-nondirectory idris-interpreter-path))
+                              "idris-history.eld")))
+
 (defun idris-repl-read-history-filename ()
   (read-file-name "Use Idris REPL history from file: "
-                  idris-repl-history-file))
+                  (idris-repl-history-file-f)))
 
 (defun idris-repl-load-history (&optional filename)
   "Set the current Idris REPL history.
 It can be read either from FILENAME or `idris-repl-history-file' or
 from a user defined filename."
   (interactive (list (idris-repl-read-history-filename)))
-  (let ((file (or filename idris-repl-history-file)))
+  (let ((file (or filename (idris-repl-history-file-f))))
     (setq idris-repl-input-history (idris-repl-read-history file))))
 
 (defun idris-repl-read-history (&optional filename)
   "Read and return the history from FILENAME.
 The default value for FILENAME is `idris-repl-history-file'."
-  (let ((file (or filename idris-repl-history-file)))
+  (let ((file (or filename (idris-repl-history-file-f))))
     (cond ((not (file-readable-p file)) '())
           (t (with-temp-buffer
                (insert-file-contents file)
                (read (current-buffer)))))))
 
 (defun idris-repl-save-history (&optional filename history)
-  "Simply save the current Idris REPL history to a file.
+  "Save the current Idris REPL HISTORY to a FILENAME.
 When Idris is setup to always load the old history and one uses only
-one instance of idris all the time, there is no need to merge the
+one instance of Idris all the time, there is no need to merge the
 files and this function is sufficient."
   (interactive (list (idris-repl-read-history-filename)))
-  (let ((file (or filename idris-repl-history-file))
+  (let ((file (or filename (idris-repl-history-file-f)))
         (hist (or history idris-repl-input-history)))
     (unless (file-writable-p file)
       (error (format "History file not writable: %s" file)))
